#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>

#include "AsciiAPI.h"

#ifdef WIN32
#include <windows.h>
#define SIGNATURE __declspec(dllexport)
#define SNPRINTF _snprintf
#else
#define SIGNATURE
#define SNPRINTF snprintf
#endif

Utils *g_pUtils = NULL;
char *g_pszOutputFilename = NULL;
static char g_exceptionBuffer[8192];

#include "SimpleMOCoutput.logic.h"


SIGNATURE void ModuleMain(RCHANDLE treeRoot, RCHANDLE dummy, RCINT64 dummier)
{
    FILE *fp = NULL;

    fp = fopen(g_pszOutputFilename, "w");
    if (!fp) {
	SNPRINTF(g_exceptionBuffer, sizeof(g_exceptionBuffer), "Couldn't open '%s' for writing!\n", g_pszOutputFilename);
	g_pUtils->pfnThrowException(g_exceptionBuffer);
    }
    OutputTree(fp, treeRoot);
    fclose(fp);
}

SIGNATURE void ModuleShutdown(RCHANDLE treeRoot, RCHANDLE dummy, RCINT64 idxdummy)
{
    free(g_pszOutputFilename);
}

SIGNATURE void ModuleVersion(RCHANDLE treeRoot, RCHANDLE dummy, RCINT64 idxdummy)
{
    *(int *)dummy = (int) GetMinorVersionWeCanRead();
}

Callback myfunctions[] = {
    {"RC-Version", ModuleVersion},
    {"DataInterChange", ModuleMain},
    {"RC-Shutdown", ModuleShutdown},
    {NULL,NULL}
};

SIGNATURE Callback *getAndSetCallbacks(Utils *pUtils, char *pszOutputASCIIfilename, char *dummy)
{
    g_pUtils = pUtils;
#ifdef _WIN32
    g_pszOutputFilename = _strdup(pszOutputASCIIfilename);
#else
    g_pszOutputFilename = strdup(pszOutputASCIIfilename);
#endif
    return myfunctions;
}


void OutputHEADER(FILE *fp, HEADER *pHEADER)
{
    fprintf(fp, "10");

    // Output StringField SENDER
    if (pHEADER->bSENDER_Exists)
        fprintf(fp, "%*s", 5, pHEADER->SENDER);
    else
        fprintf(fp, "%*s", 5, "");

    // Output StringField RECIPIENT
    if (pHEADER->bRECIPIENT_Exists)
        fprintf(fp, "%*s", 5, pHEADER->RECIPIENT);
    else
        fprintf(fp, "%*s", 5, "");

    // Output StringField FSN
    if (pHEADER->bFSN_Exists)
        fprintf(fp, "%*s", 5, pHEADER->FSN);
    else
        fprintf(fp, "%*s", 5, "");

    // Output DateField FILE_CREATION_DATE
    if (pHEADER->bFILE_CREATION_DATE_Exists)
    {
        char tmpFILE_CREATION_DATE[6+1];
        strftime(tmpFILE_CREATION_DATE, 6+1, "%y%m%d", &pHEADER->FILE_CREATION_DATE);
        fprintf(fp, "%*s", 6, tmpFILE_CREATION_DATE);
    }
    else
        fprintf(fp, "%*s", 6, "");
    fprintf(fp, "\n");
}

void OutputMOC(FILE *fp, MOC *pMOC)
{
    fprintf(fp, "20");

    // Output StringField IMSI
    if (pMOC->bIMSI_Exists)
        fprintf(fp, "%*s", 15, pMOC->IMSI);
    else
        fprintf(fp, "%*s", 15, "");

    // Output StringField CALLED_NUMBER
    if (pMOC->bCALLED_NUMBER_Exists)
        fprintf(fp, "%*s", 21, pMOC->CALLED_NUMBER);
    else
        fprintf(fp, "%*s", 21, "");

    // Output NumberField NATIONAL_IND
    if (pMOC->bNATIONAL_IND_Exists)
        fprintf(fp, "%0*d", 1, pMOC->NATIONAL_IND);
    else
        fprintf(fp, "%*s", 1, "");

    // Output DateField CALL_DATE_TIME
    if (pMOC->bCALL_DATE_TIME_Exists)
    {
        char tmpCALL_DATE_TIME[14+1];
        strftime(tmpCALL_DATE_TIME, 14+1, "%Y%m%d%H%M%S", &pMOC->CALL_DATE_TIME);
        fprintf(fp, "%*s", 14, tmpCALL_DATE_TIME);
    }
    else
        fprintf(fp, "%*s", 14, "");

    // Output NumberField DURATION
    if (pMOC->bDURATION_Exists)
        fprintf(fp, "%0*d", 7, pMOC->DURATION);
    else
        fprintf(fp, "%*s", 7, "");

    // Output StringField MSCID
    if (pMOC->bMSCID_Exists)
        fprintf(fp, "%*s", 15, pMOC->MSCID);
    else
        fprintf(fp, "%*s", 15, "");

    // Output StringField CALLEDCOUNTRYCODE
    if (pMOC->bCALLEDCOUNTRYCODE_Exists)
        fprintf(fp, "%*s", 5, pMOC->CALLEDCOUNTRYCODE);
    else
        fprintf(fp, "%*s", 5, "");

    // Output NumberField CHARGE
    if (pMOC->bCHARGE_Exists)
        fprintf(fp, "%0*d", 9, (int)(0.5+1000.0*pMOC->CHARGE));
    else
        fprintf(fp, "%*s", 9, "");

    // Output NumberField CHARGEABLE_UNITS
    if (pMOC->bCHARGEABLE_UNITS_Exists)
        fprintf(fp, "%0*d", 6, pMOC->CHARGEABLE_UNITS);
    else
        fprintf(fp, "%*s", 6, "");

    // Output NumberField EXCHANGE_RATE
    if (pMOC->bEXCHANGE_RATE_Exists)
        fprintf(fp, "%0*d", 8, (int)(0.5+100000.0*pMOC->EXCHANGE_RATE));
    else
        fprintf(fp, "%*s", 8, "");
    fprintf(fp, "\n");
}
